(function($) {
    'use strict';

    $(document).ready(tpsinitScript);

    function tpsinitScript() {

        //defing global ajax post url
        window.ajaxPostUrl = ajax_object.ajax_url;
        // validating login form request
        tpsLoginSubmit();
        // validating registration form request
        tpsRegisterSubmit();
        // validating reset password form request
        tpsResetPasswordSubmit();
        //Show Reset password
        tpsResetDisplay();
        //Return to login
        tpsLoginDisplay();
        //generateCaptcha();

    }

    // Validate login form
    function tpsLoginSubmit() {
        $('#tpsLoginForm').formValidation({
            message: 'This value is not valid',
            icon: {
                valid: 'glyphicon glyphicon-ok',
                invalid: 'glyphicon glyphicon-remove',
                validating: 'glyphicon glyphicon-refresh'
            },
            fields: {
                tps_username: {
                    message: 'The username is not valid',
                    validators: {
                        notEmpty: {
                            message: 'The username is required.'
                        }
                    }
                },
                tps_password: {
                    validators: {
                        notEmpty: {
                            message: 'The password is required.'
                        }
                    }
                }
            }
        }).on('success.form.fv', function(e) {
            $('#tps-login-alert').hide();
            // You can get the form instance
            var $loginForm = $(e.target);
            // and the FormValidation instance
            var fv = $loginForm.data('formValidation');
            var content = $loginForm.serialize();

            // start processing
            $('#tps-login-loader-info').show();
            tpsStartLoginProcess(content);
            // Prevent form submission
            e.preventDefault();
        });
    }

    // Make ajax request with user credentials
    function tpsStartLoginProcess(content) {

        var loginRequest = jQuery.ajax({
            type: 'POST',
            url: ajaxPostUrl,
            data: content + '&action=tps_user_login',
            dataType: 'json',
            success: function(data) {
                $('#tps-login-loader-info').hide();
                // check login status
                if (true == data.logged_in) {
                    $('#tps-login-alert').removeClass('alert-danger');
                    $('#tps-login-alert').addClass('alert-success');
                    $('#tps-login-alert').show();
                    $('#tps-login-alert').html(data.success);

                    // redirect to redirection url provided
                    window.location = data.redirection_url;

                } else {

                    $('#tps-login-alert').show();
                    $('#tps-login-alert').html(data.error);

                }
            },
            error: function(data) {
                console.log(data);
            }
        });
    }

    // Validate registration form


    function tpsrandomNumber(min, max) {
        return Math.floor(Math.random() * (max - min + 1) + min);
    }

    // Validate registration form
    function tpsRegisterSubmit() {
        $('#tpsRegisterForm').formValidation({
            message: 'This value is not valid',
            icon: {
                required: 'glyphicon glyphicon-asterisk',
                valid: 'glyphicon glyphicon-ok',
                invalid: 'glyphicon glyphicon-remove',
                validating: 'glyphicon glyphicon-refresh'
            },
            fields: {
                tps_fname: {
                    validators: {
                        notEmpty: {
                            message: 'The first name is required'
                        },
                        stringLength: {
                            max: 30,
                            message: 'The firstname must be less than 30 characters long'
                        },
                        regexp: {
                            regexp: /^[a-zA-Z]*$/,
                            message: 'Only characters are allowed.'
                        }
                    }
                },
                tps_username: {
                    message: 'The username is not valid',
                    validators: {
                        notEmpty: {
                            message: 'The username is required'
                        },
                        stringLength: {
                            min: 6,
                            max: 30,
                            message: 'The username must be more than 6 and less than 30 characters long'
                        },
                        regexp: {
                            regexp: /^[a-zA-Z0-9_\.]+$/,
                            message: 'The username can only consist of alphabetical, number, dot and underscore'
                        }
                    }
                },
                tps_email: {
                    validators: {
                        notEmpty: {
                            message: 'The email is required'
                        },
                        regexp: {
                            regexp: '^[^@\\s]+@([^@\\s]+\\.)+[^@\\s]+$',
                            message: 'The value is not a valid email address'
                        }
                    }
                },
                tps_password: {
                    validators: {
                        notEmpty: {
                            message: 'The password is required'
                        },
                        stringLength: {
                            min: 6,
                            message: 'The password must be more than 6 characters long'
                        }
                    }
                },
                tps_password2: {
                    validators: {
                        notEmpty: {
                            message: 'The password is required'
                        },
                        identical: {
                            field: 'tps_password',
                            message: 'The password and its confirm are not the same'
                        },
                        stringLength: {
                            min: 6,
                            message: 'The password must be more than 6 characters long'
                        }
                    }
                },
                tps_captcha: {
                    validators: {
                        callback: {
                            message: 'Wrong answer',
                            callback: function(value, validator, $field) {
                                var ans = $("#tps_captcha").attr("data-ans");
                                    //var items = $('#captchaOperation').html().split(' '),
                                    //sum = parseInt(items[0]) + parseInt(items[2]);
                                return value == ans;
                            }
                        }
                    }
                }
            }
        }).on('success.form.fv', function(e) {
            $('#tps-register-alert').hide();
            $('#tps-mail-alert').hide();
            $('body, html').animate({
                scrollTop: 0
            }, 'slow');
            // You can get the form instance
            var $registerForm = $(e.target);
            // and the FormValidation instance
            var fv = $registerForm.data('formValidation');
            var content = $registerForm.serialize();

            // start processing
            $('#tps-reg-loader-info').show();
            tpsStartRegistrationProcess(content);
            // Prevent form submission
            e.preventDefault();
        }).on('err.form.fv', function(e) {
            // Regenerate the captcha
           // generateCaptcha();
        });
    }


    // Make ajax request with user credentials
    function tpsStartRegistrationProcess(content) {

        var registerRequest = $.ajax({
            type: 'POST',
            url: ajaxPostUrl,
            data: content + '&action=tps_user_registration',
            dataType: 'json',
            success: function(data) {

                $('#tps-reg-loader-info').hide();
                //check mail sent status
                if (data.mail_status == false) {

                    $('#tps-mail-alert').show();
                    $('#tps-mail-alert').html('Could not able to send the email notification.');
                }
                // check login status
                if (true == data.reg_status) {
                    $('#tps-register-alert').removeClass('alert-danger');
                    $('#tps-register-alert').addClass('alert-success');
                    $('#tps-register-alert').show();
                    $('#tps-register-alert').html(data.success);

                } else {
                    $('#tps-register-alert').addClass('alert-danger');
                    $('#tps-register-alert').show();
                    $('#tps-register-alert').html(data.error);

                }
            },
            error: function(data) {
                console.log(data);
            }
        });
    }

    function tpsResetDisplay() {
        jQuery(document).on("click","#btnForgotPassword",function(e) { 
              $('#tpsResetPasswordSection').removeClass('hidden');
              $('#tpsLoginForm').hide();  
               $('#tpsResetPasswordSection').show();
        });
    }
    
    function tpsLoginDisplay() {
        jQuery(document).on("click","#btnReturnToLogin",function(e) { 
              $('#tpsResetPasswordSection').hide();              
              $('#tpsResetPasswordSection').addClass('hidden');
              $('#tpsLoginForm').removeClass('hidden');
              $('#tpsLoginForm').show();               
        });
    }

    // Validate reset password form
    //Neelkanth
    function tpsResetPasswordSubmit() {

        $('#tpsResetPasswordForm').formValidation({
            message: 'This value is not valid',
            icon: {
                required: 'glyphicon glyphicon-asterisk',
                valid: 'glyphicon glyphicon-ok',
                invalid: 'glyphicon glyphicon-remove',
                validating: 'glyphicon glyphicon-refresh'
            },
            fields: {
                tps_rp_email: {
                    validators: {
                        notEmpty: {
                            message: 'Please enter your email address which you used during registration.'
                        },
                        regexp: {
                            regexp: '^[^@\\s]+@([^@\\s]+\\.)+[^@\\s]+$',
                            message: 'The value is not a valid email address'
                        }
                    }
                },
                tps_newpassword: {
                    validators: {
                        notEmpty: {
                            message: 'The password is required'
                        },
                        stringLength: {
                            min: 6,
                            message: 'The password must be more than 6 characters long'
                        }
                    }
                }
            }
        }).on('success.form.fv', function(e) {
            $('#tps-resetpassword-alert').hide();

            $('body, html').animate({
                scrollTop: 0
            }, 'slow');
            // You can get the form instance
            var $resetPasswordForm = $(e.target);
            // and the FormValidation instance
            var fv = $resetPasswordForm.data('formValidation');
            var content = $resetPasswordForm.serialize();
            
            // start processing
            $('#tps-resetpassword-loader-info').show();
            tpsStartResetPasswordProcess(content);
            // Prevent form submission
            e.preventDefault();
        });
    }

    // Make ajax request with email
    //Neelkanth
    function tpsStartResetPasswordProcess(content) {
        
        var resetPasswordRequest = jQuery.ajax({
            type: 'POST',
            url: ajaxPostUrl,
            data: content + '&action=tps_resetpassword',
            dataType: 'json',
            success: function(data) {
                
                $('#tps-resetpassword-loader-info').hide();
                // check login status
                if (data.success) {
                    
                    $('#tps-resetpassword-alert').removeClass('alert-danger');
                    $('#tps-resetpassword-alert').addClass('alert-success');
                    $('#tps-resetpassword-alert').show();
                    $('#tps-resetpassword-alert').html(data.success);

                } else {

                    $('#tps-resetpassword-alert').show();
                    $('#tps-resetpassword-alert').html(data.error);

                }
            },
            error: function(data) {
                console.log(data);
            }
        });
    }



})(jQuery);
