<?php
// Add the Meta Box
function tps_ticket_details_cmb() {
    add_meta_box(
        'ticket_details_cmb', // $id
        'Ticket Details', // $title 
        'tps_show_ticket_details_cmb', // $callback
        'tickets', // $page
        'normal', // $context
        'high'); // $priority
}
add_action('add_meta_boxes', 'tps_ticket_details_cmb');

// Field Array
$prefix = 'tps_';
$custom_meta_fields = array(
    array(
        'label'=> __('User Display Name','tpsupport'),
        'desc'  => __('This is the name filled by user while creating new ticket','tpsupport'),
        'id'    => $prefix.'dispname',
        'type'  => 'text'
    ),
    array(
        'label'=> __('User Email','tpsupport'),
        'desc'  => __('Email entered by user for communication while creating new ticket','tpsupport'),
        'id'    => $prefix.'email',
        'type'  => 'text'
    ),
    array(
        'label' => __('Relevant URL','tpsupport'),
        'desc'  => __('URL filled by user while creating new ticket','tpsupport'),
        'id'    => $prefix.'url',
        'type'  => 'text'
    ),
    array(
        'label' => __('Attachments','tpsupport'),
        'desc' => __('Additional Files attached by user with ticket','tpsupport'),
        'id'    => $prefix.'attachments',
        'type'  => 'multi_attachments'
    ),
    array(
        'label' => __('Assign Agent or Agent Group','tpsupport'),
        'desc' => __('Assign this ticket to a agent','tpsupport'),
        'id'    => $prefix.'assign',
        'type'  => 'sel_users'
    ),
    array(
        'label' => __('Ticket Status','tpsupport'),
        'desc' => __('Current ticket status','tpsupport'),
        'id'    => $prefix.'status',
        'type'  => 'select',
        'options' => array (
            'new' => array (
                'label' => __('New','tpsupport'),
                'value' => 'new'
            ),
            'open' => array (
                'label' => __('Open','tpsupport'),
                'value' => 'open'
            ),
            'pending' => array (
                'label' => __('Pending','tpsupport'),
                'value' => 'pending'
            ),
            /*'onhold' => array (
                'label' => __('On Hold','tpsupport'),
                'value' => 'onhold'
            ),*/
            'solved' => array (
                'label' => __('Solved','tpsupport'),
                'value' => 'solved'
            ),
            'closed' => array (
                'label' => __('Closed','tpsupport'),
                'value' => 'closed'
            ),
        )
    ),
    array(
        'label' => __('Ticket Replies','tpsupport'),
        'desc' => __('Discussion on ticket','tpsupport'),
        'id'    => $prefix.'comments',
        'type'  => 'comments'
    ),
    array(
        'label' => __('Ticket History','tpsupport'),
        'desc' => '',
        'id'    => $prefix.'ticket_history',
        'type'  => 'repeatable'
    ),
    array(
        'label' => __('Agents in Conversation','tpsupport'),
        'desc' => __('List of all agents in this ticket conversation / to whom this ticket is assigned','tpsupport'),
        'id'    => $prefix.'allagents',
        'type'  => 'text-hidden'
    ),
    array(
        'label' => __('Last Activity','tpsupport'),
        'desc' => __('Last activity (Last status update time. All the following are covered: Recent time of ticket created, new comment on site, new comment on email, status changed or agent assigned)','tpsupport'),
        'id'    => $prefix.'last_activity',
        'type'  => 'text-hidden'
    ),
    array(
        'label' => __('Follow up Ticket ID','tpsupport'),
        'desc' => __('This ticket is a follow up of another ticket','tpsupport'),
        'id'    => $prefix.'followup',
        'type'  => 'text'
    ),

    array(
        'label' => __('Ticket Solved On','tpsupport'),
        'desc' => __('Time on which ticket was marked as solved.','tpsupport'),
        'id'    => $prefix.'ticket_solved_time',
        'type'  => 'text-hidden'
    ),
    array(
        'label' => __('User Feedback','tpsupport'),
        'desc' => __('Ticket feedback status','tpsupport'),
        'id'    => $prefix.'feedback_status',
        'type'  => 'select',
        'options' => array (
            'na' => array (
                'label' => __('Feedback Email Not sent Or Ticket Unsolved Yet','tpsupport'),
                'value' => 'na'
            ),
            'waiting' => array (
                'label' => __('Email Sent. Waiting for Feedback','tpsupport'),
                'value' => 'waiting'
            ),
            'good' => array (
                'label' => __('Rated Good','tpsupport'),
                'value' => 'good'
            ),
            'bad' => array (
                'label' => __('Rated Bad','tpsupport'),
                'value' => 'bad'
            ),
            'nofeedback' => array (
                'label' => __('No Feedback Received','tpsupport'),
                'value' => 'nofeedback'
            ),
        )
    ),

    array(
        'label' => __('User Feedback Comment','tpsupport'),
        'desc' => __('User gave a feedback comment on support received','tpsupport'),
        'id'    => $prefix.'feedback_comment',
        'type'  => 'comments'
    ),
);


// The Callback
function tps_show_ticket_details_cmb() {

  $Front_Form = new Front_Form();

  global $custom_meta_fields, $post;
  // Use nonce for verification
  echo '<input type="hidden" name="ticket_details_cmb_nonce" value="'.wp_create_nonce(basename(__FILE__)).'" />';
     
    // Begin the field table and loop
    echo '<table class="form-table">';
    foreach ($custom_meta_fields as $field) {
        // get value of this field if it exists for this post
        $meta = get_post_meta($post->ID, $field['id'], true);
       
        $field_label = "";
        if(isset($field['label'])){
          $field_label = $field['label'];
        }

        $row_hidden = false;
        if($field['type'] == "text-hidden"){
          $row_hidden = true;
        }

        // begin a table row with
        if(!$row_hidden){
            echo '<tr>
                <th style="width:20%;"><label for="'.$field['id'].'">'.$field_label.'</label></th>
                <td style="width:80%;">';
        }

                switch($field['type']) {
                    // case items will go here
                  // text
                  case 'text':
                      echo '<input type="text" name="'.$field['id'].'" id="'.$field['id'].'" value="'.$meta.'" />
                          <br /><span class="description">'.$field['desc'].'</span>';
                  break;

                  // text-hidden
                  case 'text-hidden':
                      echo '<input type="hidden" name="'.$field['id'].'" id="'.$field['id'].'" value="'.$meta.'" />';
                  break;

                  // textarea
                  case 'textarea':
                      echo '<textarea name="'.$field['id'].'" id="'.$field['id'].'" cols="60" rows="4">'.$meta.'</textarea>
                          <br /><span class="description">'.$field['desc'].'</span>';
                  break;

                  // checkbox
                  case 'checkbox':
                      echo '<input type="checkbox" name="'.$field['id'].'" id="'.$field['id'].'" ',$meta ? ' checked="checked"' : '','/>
                          <label for="'.$field['id'].'">'.$field['desc'].'</label>';
                  break;

                  // select
                  case 'select':
                      echo '<select name="'.$field['id'].'" id="'.$field['id'].'">';
                      foreach ($field['options'] as $option) {
                          echo '<option', $meta == $option['value'] ? ' selected="selected"' : '', ' value="'.$option['value'].'">'.$option['label'].'</option>';
                      }
                      echo '</select>'; 
                      if($field['id'] == "tps_status"){
                        echo "<input type='hidden' value='".$meta."' name='".$field['id']."_prev' id='".$field['id']."_prev'>";
                      }
                      echo '<br /><span class="description">'.$field['desc'].'</span>';

                  break;

                  // radio
                  case 'radio':
                      foreach ( $field['options'] as $option ) {
                          echo '<input type="radio" name="'.$field['id'].'" id="'.$option['value'].'" value="'.$option['value'].'" ',$meta == $option['value'] ? ' checked="checked"' : '',' />
                                  <label for="'.$option['value'].'">'.$option['label'].'</label><br />';
                      }
                  break;

                  // checkbox_group
                  case 'checkbox_group':
                      foreach ($field['options'] as $option) {
                          echo '<input type="checkbox" value="'.$option['value'].'" name="'.$field['id'].'[]" id="'.$option['value'].$field['id'].'"',$meta && in_array($option['value'], $meta) ? ' checked="checked"' : '',' /> 
                                  <label for="'.$option['value'].$field['id'].'">'.$option['label'].'</label><br />';
                      }
                      echo '<span class="description">'.$field['desc'].'</span>';
                  break;

                  // tax_select
                  case 'tax_select':
                      echo '<select name="'.$field['id'].'" id="'.$field['id'].'">
                              <option value="">Select One</option>'; // Select One
                      $terms = get_terms($field['id'], 'get=all');
                      
                      $selected = wp_get_object_terms($post->ID, $field['id']);
                      
                      foreach ($terms as $term) {
                          if (!empty($selected) && !strcmp($term->slug, $selected[0]->slug)) 
                              echo '<option value="'.$term->slug.'" selected="selected">'.$term->name.'</option>'; 
                          else
                              echo '<option value="'.$term->slug.'">'.$term->name.'</option>'; 
                      }
                      $taxonomy = get_taxonomy($field['id']);
                      echo '</select><br /><span class="description"><a href="'.home_url().'/wp-admin/edit-tags.php?taxonomy='.$field['id'].'">Manage '.$taxonomy->label.'</a></span>';
                  break;


                  // post_list
                  case 'post_list':
                  $items = get_posts( array (
                      'post_type' => $field['post_type'],
                      'posts_per_page' => -1
                  ));
                      echo '<select name="'.$field['id'].'" multiple id="'.$field['id'].'">
                              <option value="">Select One</option>'; // Select One
                          foreach($items as $item) {
                              echo '<option value="'.$item->ID.'"',$meta == $item->ID ? ' selected="selected"' : '','>'.$item->post_type.': '.$item->post_title.'</option>';
                          } // end foreach
                      echo '</select><br /><span class="description">'.$field['desc'].'</span>';
                  break;


                  // date
                  case 'date':
                    echo '<input type="text" class="datepicker" name="'.$field['id'].'" id="'.$field['id'].'" value="'.$meta.'" size="30" />
                        <br /><span class="description">'.$field['desc'].'</span>';
                  break;


                  // slider
                  case 'slider':
                  $value = $meta != '' ? $meta : '0';
                      echo '<div id="'.$field['id'].'-slider"></div>
                              <input type="text" name="'.$field['id'].'" id="'.$field['id'].'" value="'.$value.'" size="5" />
                              <br /><span class="description">'.$field['desc'].'</span>';
                  break;


                  // image
                  case 'image':
                      $image = plugin_dir_url( __FILE__ ).'images/image.jpg';  
                      echo '<span class="custom_default_image" style="display:none">'.$image.'</span>';
                      if ($meta) { $image = wp_get_attachment_image_src($meta, 'medium'); $image = $image[0]; }  
                     // echo $image;             
                      echo    '<input name="'.$field['id'].'" type="hidden" class="custom_upload_image" value="'.$meta.'" />
                                  <img src="'.$image.'" height="100px" width="auto"  class="custom_preview_image" alt="" /><br />
                                      <input class="custom_upload_image_button button" type="button" value="Choose Image" />
                                      <small> <a href="#" class="custom_clear_image_button">Remove Image</a></small>
                                      <br clear="all" /><span class="description">'.$field['desc'].'</span>';
                  break;

                  // repeatable
                  case 'repeatable':
                      //echo '<a class="repeatable-add button" href="javascript:;">+</a>';
                      
                      echo '<ul id="'.$field['id'].'-repeatable" class="custom_repeatable">';
                      $i = 0;
                      if ($meta) {
                          foreach($meta as $row) {
                              echo '<li>'; 
                              $exp = explode("[", $row);
                              echo "<div class='history-row'>:- ".$exp[0]."</div>";
                              echo '<span class="sort hndle hidden">|||</span>
                                          <input type="hidden" name="'.$field['id'].'['.$i.']" id="'.$field['id'].'" value="'.$row.'" size="30" />
                                          </li>';
                              //echo '<a class="repeatable-remove button" href="javascript:;">-</a>';
                              $i++;
                          }
                      } else {
                          echo __("No Ticket History Yet!","");
                        
                      }
                      echo '</ul>
                          <span class="description">'.$field['desc'].'</span>';
                  break;

				// multi_attachments
                  case 'multi_attachments':
                      echo '<a class="repeatable-add button" href="javascript:;">+</a>
                              <div class="spacer"></div><ul id="'.$field['id'].'-repeatable" class="custom_repeatable">';
                      $i = 0;
                      if ($meta) {
                          foreach($meta as $row) {
                              echo '<li class="ticket-attachment-admin"><span class="sort hndle">|||</span>';
                                          $thumb = wp_get_attachment_thumb_url($row);
                                          if($thumb){
                                            echo "<a class='media' href='".wp_get_attachment_url( $row )."' target='_blank'><span class='thumb'>"."<img src='".$thumb."'>"."</span></a>";
                                          }
                                          echo '<input type="text" name="'.$field['id'].'['.$i.']" id="'.$field['id'].'" value="'.$row.'" size="30" />
                                          <a class="repeatable-remove button" href="javascript:;">x</a></li>';
                              $i++;
                          }
                      } else {
                        echo '<li class="ticket-attachment-admin"><span class="sort hndle">|||</span>';
                                         // $thumb = wp_get_attachment_thumb_url($row);
                                         // if($thumb){
                                            echo "<a class='media addnew' href='#' target='_blank'><span class='add'>"."Fill attachment id below:"."</span></a>";
                                         // }
                                          echo '<input type="text" name="'.$field['id'].'['.$i.']" id="'.$field['id'].'" value="" size="30" />
                                          <a class="repeatable-remove button" href="javascript:;">x</a></li>';
                         
                      }
                      echo '</ul><div class="spacer"></div>
                          <span class="description">'.$field['desc'].'</span>';

                  break;

				// comments
                  case 'comments':
                      
                      echo '<ul id="'.$field['id'].'-rcomment" class="custom_rcomment">';
                      $i = 0;
                      if ($meta) {
                              //echo "<pre>"; print_r($meta); echo "</pre>";
                          foreach($meta as $rowk => $row) {
                          	  //echo "<pre>"; print_r($row); echo "</pre>";
                              if(!isset($row["ticket_id"])){$row["ticket_id"] = "";}
                              if(!isset($row["from"])){$row["from"] = "";}
                              if(!isset($row["to"])){$row["to"] = "";}
                              if(!isset($row["date"])){$row["date"] = "";}
                              if(!isset($row["udate"])){$row["udate"] = "";}
                              if(!isset($row["in_reply_to"])){$row["in_reply_to"] = "";}
                              if(!isset($row["uid"])){$row["uid"] = "";}
                              if(!isset($row["msgno"])){$row["msgno"] = "";}
                              if(!isset($row["subject"])){$row["subject"] = "";}
	                        if(is_array($row)){
                              echo "<li> 
                              <div class='rcomm-display 1111'>";
                              echo '<a class="rcomment-remove button" href="javascript:;">x</a>';
                               tps_comment_author_details($row,$Front_Form);
                               
                               $viewstr = $row["content"];
                              
                               //echo $viewstr;
                              echo "<article>".wpautop($viewstr)."</article>";
                                echo "<div class='attach'>";
                                if(isset($row['attachments']) && is_array($row['attachments']) && sizeof($row['attachments']) > 0){
                                    foreach($row['attachments'] as $attachid){
                                      $thumb = wp_get_attachment_thumb_url($attachid);
                                          if($thumb){
                                            echo "<a class='media' href='".wp_get_attachment_url( $attachid )."' target='_blank'><span class='thumb'>"."<img src='".$thumb."'>"."</span></a>";
                                          }
                                    echo "<input role='comment-attachment' type='hidden' name='".$field["id"]."[".$rowk."][attachments][]' id='".$field["id"]."' value='".$attachid."'>";
                                    }
                                    //echo "</div>";
                                }

                              echo "</div>

  

                              <textarea class='hidden' role='comment-text' name='".$field["id"]."[".$rowk."][content]' id='".$field["id"]."'>".$row["content"]."
                                          </textarea>
                                          <input class='hidden' role='comment-userid' name='".$field["id"]."[".$rowk."][userId]' id='".$field["id"]."' value='".$row["userId"]."'>";
                                          echo "<input type='hidden' class='text' role='comment-ticketid' name='".$field["id"]."[".$rowk."][ticket_id]' id='".$field["id"]."' value='".$row["ticket_id"]."'>";
                                          echo "<input type='hidden' class='text' role='comment-from' name='".$field["id"]."[".$rowk."][from]' id='".$field["id"]."' value='".$row["from"]."'>";
                                          echo "<input type='hidden' class='text' role='comment-to' name='".$field["id"]."[".$rowk."][to]' id='".$field["id"]."' value='".$row["to"]."'>";
                                          echo "<input type='hidden' class='text' role='comment-date' name='".$field["id"]."[".$rowk."][date]' id='".$field["id"]."' value='".$row["date"]."'>";
                                          echo "<input type='hidden' class='text' role='comment-udate' name='".$field["id"]."[".$rowk."][udate]' id='".$field["id"]."' value='".$row["udate"]."'>";
                                          echo "<input type='hidden' class='text' role='comment-in_reply_to' name='".$field["id"]."[".$rowk."][in_reply_to]' id='".$field["id"]."' value='".$row["in_reply_to"]."'>";
                                          echo "<input type='hidden' class='text' role='comment-uid' name='".$field["id"]."[".$rowk."][uid]' id='".$field["id"]."' value='".$row["uid"]."'>";
                                          echo "<input type='hidden' class='text' role='comment-msgno' name='".$field["id"]."[".$rowk."][msgno]' id='".$field["id"]."' value='".$row["msgno"]."'>";
                                          echo "<input type='hidden' class='text' role='comment-subject' name='".$field["id"]."[".$rowk."][subject]' id='".$field["id"]."' value='".$row["subject"]."'>";
            
                               echo '</div></li>';
                              $i++;
	                        }

                          }
                      } else {
                          echo __("No Comment Yet!","");
                         
                      }
                      echo '</ul>
                          <span class="description">'.$field['desc'].'</span>';

                  break;

                  // sel_users
                  case 'sel_users':

                          tps_select_users_dropdown($field,$meta,"");
                      echo "<input type='hidden' value='".$meta."' name='".$field['id']."_prev' id='".$field['id']."_prev'>";
                      echo '<br /><span class="description">'.$field['desc'].'</span>';
                  break;



                } //end switch

      if(!$row_hidden){
        echo '</td></tr>';
      }
    } // end foreach
    echo '</table>'; // end table
}

function tps_select_users_dropdown($field = "",$meta = "",$display = ""){

                      $agents_arr = tps_get_all_entities_arr('agents');
                      $agentgroups_arr = tps_get_all_entities_arr('groups');
                      $agentgroups_admin_arr = tps_get_all_entities_arr('groupsadmin');
                      $agentsadmin_arr = tps_get_all_entities_arr('agentadmin');

//                    echo "<pre>"; print_r($assign_arr); echo "</pre>";

                      if($display == "frontend"){
                        echo "<select name='assign_agent' id='assign_agent'><option value='0'>".__("Select anyone","tpsupport")."</option>"; 
                      } else {
                        echo "<select name='".$field['id']."' id='".$field['id']."'><option value='0'>".__("Select anyone","tpsupport")."</option>"; 
                      }

                      echo "<optgroup label='".__("Select Agent","tpsupport")."'>";
                      foreach ($agents_arr as $id => $name) {
                          $selected = "";
                          if($meta == $id){
                            $selected = "selected='selected'";
                          }
                          echo "<option value='".$id."' ".$selected.">".$name."</option>";
                      }
                      echo "</optgroup>";

                      echo "<optgroup label='".__("Select Agent Group","tpsupport")."'>";
                      foreach ($agentgroups_arr as $id => $name) {
                          $selected = "";
                          if($meta == $id){
                            $selected = "selected='selected'";
                          }
                          echo "<option value='".$id."' ".$selected.">".$name."</option>";
                      }
                      echo "</optgroup>";

                      echo "<optgroup label='".__("Select Agent Group Admins","tpsupport")."'>";
                      foreach ($agentgroups_admin_arr as $id => $name) {
                          $selected = "";
                          if($meta == $id){
                            $selected = "selected='selected'";
                          }
                          echo "<option value='".$id."' ".$selected.">".$name."</option>";
                      }
                      echo "</optgroup>";

                      echo "<optgroup label='".__("Select Agent Admin","tpsupport")."'>";
                      foreach ($agentsadmin_arr as $id => $name) {
                          $selected = "";
                          if($meta == $id){
                            $selected = "selected='selected'";
                          }
                          echo "<option value='".$id."' ".$selected.">".$name."</option>";
                      }
                      echo "</optgroup>";

                      echo "</select>";
}


// Save the Data
function tps_save_ticket_details_cmb($post_id) {
    global $custom_meta_fields;
  
      $Front_Form = new Front_Form();
      $time = $Front_Form->tps_get_current_time('ts');     
      $current_user = wp_get_current_user();
      $userId = $current_user->ID;
      $user_name = $current_user->display_name;
      $history_status = "";
      $history_assign = "";

    // verify nonce
    if(isset($_POST)){
    	if (!isset($_POST['ticket_details_cmb_nonce']) || !wp_verify_nonce($_POST['ticket_details_cmb_nonce'], basename(__FILE__))){ 
          return $post_id;
    	}
	}

    // check autosave
    if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE)
        return $post_id;
    // check permissions
    if ('page' == $_POST['post_type']) {
        if (!current_user_can('edit_page', $post_id))
            return $post_id;
        } elseif (!current_user_can('edit_post', $post_id)) {
            return $post_id;
    }
     
    // loop through fields and save the data
    foreach ($custom_meta_fields as $field) {
        if($field['type'] == 'tax_select') {continue;}
    		
        
        if($field['type'] == 'comments'){
    		
    		}  
        
        $old = get_post_meta($post_id, $field['id'], true);
        $new = $_POST[$field['id']];

        if($field['type'] == 'multi_attachments'){
          $new = array_filter(array_values($new));
         
        }  



        // manage status history
        if($field['id'] == "tps_status"){
          $prev = $_POST[$field['id']."_prev"];
          if($new != $prev){
            $history_status = "Status Changed to ".$new." from ".$prev." by user: ".$user_name." on: ".$time." from admin panel [userId:".$userId."|||user_name:".$user_name."|||status:".$new."|||status_from:".$prev."|||time:".$time."|||location:admin]";

          }
          
        }
        
        // manage assign history
        if($field['id'] == "tps_assign"){
          $prev = $_POST[$field['id']."_prev"];
          if($new != $prev){
            $history_assign = "Assigned to: ".$new." from ".$prev." by user: ".$user_name." on: ".$time." from admin panel [userId:".$userId."|||user_name:".$user_name."|||assigned_to:".$new."|||assigned_from:".$prev."|||time:".$time."|||location:admin]";
          }
          
        }




        // manage all agents history list
        if($field['id'] == "tps_allagents"){
          $new = $new."|".$_POST['tps_assign']."|";
        }

        // if status is solved, then change ticket solved time
        if($field['id'] == "tps_ticket_solved_time"){
          //$new = $_POST[$field['id']];
          $check_status = $_POST['tps_status'];
          if($check_status == "solved"){
            $new = $time;
          }
        }



        if ($new && $new != $old) {
            update_post_meta($post_id, $field['id'], $new);
        } elseif ('' == $new && $old) {
            delete_post_meta($post_id, $field['id'], $old);
        }
    } // end foreach

      if($history_status != ""){
        $update_history1 = $Front_Form->tps_update_ticket_history($post_id,$history_status);
      }

      if($history_assign != ""){
          $update_history2 = $Front_Form->tps_update_ticket_history($post_id,$history_assign);
      }


}
add_action('save_post', 'tps_save_ticket_details_cmb');





// Add the Meta Box
function tps_ticket_agentgroups_cmb() {
    add_meta_box(
        'ticket_agentgroups_cmb', // $id
        'Agent Users Group Details', // $title 
        'tps_show_ticket_agentgroups_cmb', // $callback
        'agentgroups', // $page
        'normal', // $context
        'high'); // $priority
}
add_action('add_meta_boxes', 'tps_ticket_agentgroups_cmb');



// Field Array
$prefix = 'tpsag_';
$agents_arr = tps_get_all_entities_arr();
$cmb_tickets_agentgroups_fields = array(
    
   
    array(
        'label'=> __('Select Agents in this group','tpsupport'),
        'desc'  => '',
        'id'    => $prefix.'allagents',
        'type'  => 'agentuser_group',
        'options' => $agents_arr,
    ),
);



function tps_get_all_entities_arr($ret = "agents",$alloption = "",$defaultnull = "",$format = ""){


    // loop through all agents
    $agents_arr = array();
    if($defaultnull == true){
      $agents_arr[] = __("Select","tpsupport");
    }
    if($alloption == "all"){
      $agents_arr['all'] = __("Select All","tpsupport");
    }

    if($ret == "agents" || $ret == "all"){

        $agentsargs = array(
          'role'         => 'agent',
          'fields'       => array('ID','display_name')
         ); 
        $agents_obj = get_users( $agentsargs );
        if($format == "id"){
            foreach ($agents_obj as $userobj) {
              $agents_arr[] = $userobj->ID;
            }
        } else if($format == "idname"){
            foreach ($agents_obj as $userobj) {
              $agents_arr[$userobj->ID] = array("name" => $userobj->display_name);
            }
        } else {
            foreach ($agents_obj as $userobj) {
              $agents_arr[$userobj->ID] = $userobj->display_name;
            }
        }
       
    }


    // loop through all agent admins
    $agentadmin_arr = array();
    if($defaultnull == true){
      $agentadmin_arr[] = __("Select","tpsupport");
    }
    if($alloption == "all"){
      $agentadmin_arr['all'] = __("Select All","tpsupport");
    }

    if($ret == "agentadmin" || $ret == "all"){

        $agentsadmin_args = array(
          'role'         => 'agentadmin',
          'fields'       => array('ID','display_name')
         ); 
        $agentsadmin_obj = get_users( $agentsadmin_args );
        
        if($format == "id"){
            foreach ($agentsadmin_obj as $userobj) {
              $agentadmin_arr[] = $userobj->ID;
            }
        } else if($format == "idname"){
            foreach ($agentsadmin_obj as $userobj) {
              $agentadmin_arr[$userobj->ID] = array("name" => $userobj->display_name);
            }
        } else{
            foreach ($agentsadmin_obj as $userobj) {
              $agentadmin_arr[$userobj->ID] = $userobj->display_name;
            }
        }
        
    }


    // loop through all agent groups

    $agent_groups_arr = array();
    if($defaultnull == true){
      $agent_groups_arr[] = __("Select","tpsupport");
    }
    if($alloption == "all"){
      $agent_groups_arr['all'] = __("Select All","tpsupport");
    }
    
    if($ret == "groups" || $ret == "all"){
        $loopgroups = get_posts( array (
              'post_type' => 'agentgroups',
              'posts_per_page' => -1,
              'order' => 'ASC',
              'orderby' => 'title'
        ));
        
        if($format == "id"){
              foreach($loopgroups as $agobj){
                    $agid = $agobj->ID;
                    $agent_groups_arr[] = "G-".$agid;
                }
        } else if($format == "idname"){
              foreach($loopgroups as $agobj){
                    $agtitle = $agobj->post_title;
                    $agid = $agobj->ID;
                    $agent_groups_arr["G-".$agid] = array("name" => $agtitle);
                }  
        } else {
              foreach($loopgroups as $agobj){
                    $agtitle = $agobj->post_title;
                    $agid = $agobj->ID;
                    $agent_groups_arr["G-".$agid] = $agtitle;
                }  
        }
        
    }
    

    // loop through all agent groups having at least one group admin
    $agent_groups_admin_arr = array();
    if($defaultnull == true){
      $agent_groups_admin_arr[] = __("Select","tpsupport");
    }
    if($alloption == "all"){
      $agent_groups_admin_arr['all'] = __("Select All","tpsupport");
    }
    
    if($ret == "groupsadmin" || $ret == "all"){
        $loopgroups = get_posts( array (
              'post_type' => 'agentgroups',
              'posts_per_page' => -1,
              'order' => 'ASC',
              'orderby' => 'title'
        ));
       
        foreach($loopgroups as $agobj){

                  $agtitle = $agobj->post_title." ".__("Admin","tpsupport");
                  $agid = $agobj->ID;
                  $agadmins = get_post_meta($agid,"tpsag_allagents",true);
                  //echo "<pre>"; print_r($agadmins); echo "</pre>";

                  if(isset($agadmins['admin']) && sizeof($agadmins['admin']) > 0){

                    if($format == "id"){
                        $agent_groups_admin_arr[] = "GA-".$agid;
                      } else if($format == "idname"){
                          $agent_groups_admin_arr["GA-".$agid] = array("name" => $agtitle);
                      } else {
                          $agent_groups_admin_arr["GA-".$agid] = $agtitle;
                      }

                  }
        }
        //die;
    }



// loop through all clients
    $clients_arr = array();
    if($defaultnull == true){
      $clients_arr[] = __("Select","tpsupport");
    }
    if($alloption == "all"){
      $clients_arr['all'] = __("Select All","tpsupport");
    }

    if($ret == "clients" || $ret == "all"){

        $clientsargs = array(
          'role'         => 'client',
          'fields'       => array('ID','display_name')
         ); 
        $clients_obj = get_users( $clientsargs );
        if($format == "id"){
            foreach ($clients_obj as $userobj) {
              $clients_arr[] = $userobj->ID;
            }
        } else if($format == "idname"){
            foreach ($clients_obj as $userobj) {
              $clients_arr[$userobj->ID] = array("name" => $userobj->display_name);
            }
        } else {
            foreach ($clients_obj as $userobj) {
              $clients_arr[$userobj->ID] = $userobj->display_name;
            }
        }
       
    }








    if($ret == "groups"){
      return $agent_groups_arr;
    
    } else if($ret == "groupsadmin"){
      return $agent_groups_admin_arr;
    
    } else if($ret == "agents"){
        return $agents_arr;
    
    } else if($ret == "agentadmin"){
        return $agentadmin_arr;
    
    } else if($ret == "clients"){
        return $clients_arr;
    
    } else if($ret == "all"){
      return array_replace($agents_arr, $agent_groups_arr);
    
    } 

    return $agents_arr;
}




// The Callback
function tps_show_ticket_agentgroups_cmb() {
  global $cmb_tickets_agentgroups_fields, $post;
  // Use nonce for verification
  echo '<input type="hidden" name="ticket_agentgroups_cmb_nonce" value="'.wp_create_nonce(basename(__FILE__)).'" />';
     
    // Begin the field table and loop
    echo '<table class="form-table">';
    foreach ($cmb_tickets_agentgroups_fields as $field) {
        // get value of this field if it exists for this post
        $meta = get_post_meta($post->ID, $field['id'], true);
   
        $field_label = "";
        if(isset($field['label'])){
          $field_label = $field['label'];
        }
        // begin a table row with
        echo '<tr>
                <th><label for="'.$field['id'].'">'.$field_label.'</label></th>
                <td>';
                switch($field['type']) {
                    // case items will go here
                 
                  // agentuser_group
                  case 'agentuser_group':
                    echo "<table class='table_agent_groups'><tr><th>".__("Agent","tpsupport")."</th><th>".__("Select","tpsupport")."</th><th>".__("Group Admin","tpsupport")."</th>";
                      foreach ($field['options'] as $userid => $display_name) {

                        $disabled = "";
                         if( !isset($meta['agents']) || !is_array($meta['agents']) || !in_array($userid, $meta['agents'])){
                            $disabled = "disabled";
                         } 

                          echo '<tr><td> 
                                  <label for="'.$userid.$field['id'].'">'.$display_name.'</label></td><td>
                          <input type="checkbox" class="agentslist" value="'.$userid.'" name="'.$field['id'].'[]" id="'.$userid.$field['id'].'"', ($meta && is_array($meta['agents']) && in_array($userid, $meta['agents'])) ? ' checked="checked"' : '',
                          '/>

                          </td><td><input type="checkbox" class="adminslist admin-'.$userid.'" value="'.$userid.'" name="'.$field['id'].'_admin[]" id="'.$userid.$field['id'].'_admin"', ($meta && is_array($meta['admin']) && in_array($userid, $meta['admin'])) ? ' checked="checked"' : '',

                                  ' '.$disabled.'/></td></tr>';
                      }
                      echo '</table><span class="description">'.$field['desc'].'</span>';
                  break;

                } //end switch
        echo '</td></tr>';
    } // end foreach
    echo '</table>'; // end table
}


// Save the Data
function tps_save_ticket_agentgroups_cmb($post_id) {
    global $cmb_tickets_agentgroups_fields;
     
    // verify nonce
    if(isset($_POST)){
      if (!isset($_POST['ticket_agentgroups_cmb_nonce']) || !wp_verify_nonce($_POST['ticket_agentgroups_cmb_nonce'], basename(__FILE__))){ 
          return $post_id;
      }
     }

    // check autosave
    if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE)
        return $post_id;
    // check permissions
    if ('page' == $_POST['post_type']) {
        if (!current_user_can('edit_page', $post_id))
            return $post_id;
        } elseif (!current_user_can('edit_post', $post_id)) {
            return $post_id;
    }
     
    // loop through fields and save the data
    foreach ($cmb_tickets_agentgroups_fields as $field) {
        if($field['type'] == 'tax_select') {continue;}
        
        $old = get_post_meta($post_id, $field['id'], true);
        $new = $_POST[$field['id']];

        if($field['id'] == "tpsag_allagents"){
         $new = array();
         $new['admin'] = $_POST[$field['id']."_admin"];
         if(sizeof($new['admin'])){
           $all_agents = array_unique(array_merge($_POST[$field['id']],$new['admin']));
         } else {
          $all_agents = $_POST[$field['id']];
         }
         $new['agents'] = $all_agents;
         //echo "<pre>"; print_r($new['admin']); print_r($_POST[$field['id']]);  print_r($new);
         //die;
        }

        if ($new && $new != $old) {
            update_post_meta($post_id, $field['id'], $new);
        } elseif ('' == $new && $old) {
            delete_post_meta($post_id, $field['id'], $old);
        }
    } // end foreach


}
add_action('save_post', 'tps_save_ticket_agentgroups_cmb');






add_action( 'current_screen', 'tps_get_screen', 0 );

function tps_get_screen(){
	if(is_admin()) {
		$screen = get_current_screen();
		//echo "<pre>"; print_r($screen);echo "</pre>"; die; 
		if(isset($screen->post_type) && ($screen->post_type == "tickets" || $screen->post_type == "agentgroups" || in_array( $screen->id, array("toplevel_page_tps_settings","admin_page_tps_settings_email"))
  )){
			    //wp_enqueue_script('jquery-ui-datepicker');
			    // wp_enqueue_style('jquery-ui-custom', plugin_dir_url( __FILE__ ).'css/jquery-ui.css');
			    // wp_enqueue_script('jquery-ui-slider');
		    wp_enqueue_script('custom-js', plugin_dir_url( __FILE__ ).'../js/custom-js.js');

        $url = plugins_url('/', __FILE__).'../css/style-admin.css';
        wp_deregister_style('tpsadmin');
        wp_register_style('tpsadmin', $url);
        wp_enqueue_style('tpsadmin');


		}
	}
}

?>